/*
* SocketTunnel - Socket Proxies to create services proxies that can 
* be controlled by application test suites.
*
* Copyright (C) 2004  Abhilash Koneri
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.sf.jshell2;

import java.io.*;
import javax.net.ssl.*;
import java.security.*;
import java.util.*;

/*******************
 * The SPI for the jshell initializes a simple key Manager. The default 
 * key manager in the JSSE is obtained and set up with the custom key manager.
 * The following system properties are queried to set up the custom key manager
 * <ol>
 * <li><code>jshell.default.keymanager.alg</code>: The default key manager algorithm in the JSSE provider 
 *   (For SunJVM it is SunX509, for IBM JVM it is IbmX509).
 * <li><code>jshell.server_key_map.file</code>: Path to the config file containing the server to key mapping
 * </ol>
 *
 * The custom key manager is initialized with
 * <ol>
 *     <li>An intance of the default key manager provided by the platform.
 *     <li>A mapping of the server names to the key alias.
 * </ol>
 * 
 * @author Abhilash Koneri
 * @since nothing 0.0
 */
public class JSKeyManagerFactorySpi  extends KeyManagerFactorySpi
{
    private JSKeyManager keyManager = null;

    public KeyManager[] engineGetKeyManagers()
    {
        System.out.println("JSKeyManagerFactorySpi::engineGetKeyManagers()");
        return new KeyManager[]{ keyManager};
    }

    public void engineInit(KeyStore ks, char[] password) 
        throws KeyStoreException, NoSuchAlgorithmException,  UnrecoverableKeyException
    {
        System.out.println("JSKeyManagerFactorySpi::engineInit(ks)");

        String defaultAlg = System.getProperty("jshell.default.keymanager.alg");
        if(defaultAlg == null ||
                defaultAlg.trim().length() == 0)
        {
            throw new NoSuchAlgorithmException("The system property jshell.default.keymanager.alg "+
                    "should be set to the JVM's default keymanagment algorithm");
        }
        String keyMapFile = System.getProperty("jshell.server_key_map.file");

        Properties serverMap = new Properties();
        FileInputStream inputStream = null;
        try
        {
            inputStream = new FileInputStream(new File(keyMapFile));
            serverMap.load(inputStream);
        }
        catch(Exception e)
        {
            throw new UnrecoverableKeyException("Cannot read key map file "+keyMapFile);
        }
        finally
        {
            if(inputStream != null)
            {
                try
                {
                    inputStream.close();
                }
                catch(Exception e)
                {
                }
            }
        }

        System.out.println(serverMap);


        KeyManagerFactory factory = KeyManagerFactory.getInstance(defaultAlg);
        factory.init(ks, password);

        KeyManager[] keyManagers = factory.getKeyManagers();

        if(keyManagers ==  null ||
         keyManagers.length == 0)
        {
            throw new NoSuchAlgorithmException("The default algorithm :"+
                    defaultAlg+" produced no key managers");
        }
        X509KeyManager x509KeyManager = null;

        for(int i=0;i<keyManagers.length; i++)
        {
            if(keyManagers[i] instanceof X509KeyManager)
            {
                x509KeyManager = (X509KeyManager)keyManagers[i];
                break;
            }
        }

        if(x509KeyManager == null)
        {
            throw new NoSuchAlgorithmException("The default algorithm :"+
                    defaultAlg+" did not produce a X509 Key manager");
        }
            
        this.keyManager = new JSKeyManager(x509KeyManager, serverMap);
    }

    public void engineInit(ManagerFactoryParameters spec)
    {
        System.out.println("JSKeyManagerFactorySpi::engineInit(spec)");
    }

    private String[] parseList(String s)
    {
        StringTokenizer tokenizer = new StringTokenizer(s, ",");
        ArrayList list = new ArrayList();
        while(tokenizer.hasMoreTokens())
        {
            list.add(tokenizer.nextToken());
        }
        return (String[])list.toArray(new String[0]);
    }
}
